/*
 *	adsbox - ADS-B decoder software
 *	(c) atty 2011-17 romulmsg@mail.ru
 *
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <pthread.h>
#include "misc.h"
#include "db.h"

#ifdef MLAT
#include <mlat.h>
#endif

unsigned int avr_data (adsb_data *data, char *b) {
	char timestamp[9];

	bzero(data->avr_data, sizeof(data->avr_data));
	strncpy(data->avr_data, b, sizeof(data->avr_data));

	//DEBUG("\navr_data() %s len is %u\n", data->avr_data, strlen(data->avr_data));

	data->timestamp = 0;
	// strip time mark if present. Thanks Petr Kouda
	if (strlen(b) == 28 || strlen(b) == 42) {
		bzero(timestamp, sizeof(timestamp));
		strncpy(timestamp, b + 5, 8);
		data->timestamp = strtoll(timestamp, NULL, 16); // extract timestamp here
		sprintf(data->avr_data, "*%s", data->avr_data + 13);
	}
	
	if (strlen(data->avr_data) == 30) {
		data->msg_type = MSG_TYPE_LONG;
		sscanf(data->avr_data,
			"*%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx",
			&data->data[0], &data->data[1], &data->data[2], &data->data[3], &data->data[4],
			&data->data[5], &data->data[6], &data->data[7], &data->data[8], &data->data[9],
			&data->data[10], &data->data[11], &data->data[12], &data->data[13]);
		return 0;
	}
	if (strlen(data->avr_data) == 16) {
		data->msg_type = MSG_TYPE_SHORT;
		sscanf(data->avr_data,
			"*%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx%02hhx",
			&data->data[0], &data->data[1], &data->data[2], &data->data[3], &data->data[4],
			&data->data[5], &data->data[6]);
		return 0;
	}

	// telemetry (freq., noise level) packet (ADSBox AVR protocol extension)
	if (strlen(data->avr_data) == 13) {
		data->msg_type = MSG_TYPE_TELEMETRY;
		sscanf(data->avr_data,
			"#%02hhx%02hhx%02hhx%02hhx",
			&data->data[0], &data->data[1], &data->data[2], &data->data[3]);
		static unsigned int freq, nl;
		if (!sscanf(data->avr_data, "#%08X:%02X", &freq, &nl)) {
			DEBUG("\nError convert frequency and NL.");
		} else {
			db_update_hz(data->source_id, freq);
			db_update_nl(data->source_id, nl);
		}
		
		return 0;
	}

	return 1;
}

void *tcp_server(void *arg) {
	server_info *si = (server_info *) arg;
	int s, en;
	pthread_t child;

	s = socket(AF_INET6, SOCK_STREAM, 0);
	if (s < 0)
		panic("avr server socket()");
	if (setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &en, sizeof(en)) != 0)
		panic("avr server setsockopt()");

	// non-blocking socket
	if (ioctl(s, FIONBIO, &en) < 0)
		panic("avr server ioctl()");

	//--- bind  ---
	struct sockaddr_in6 addr_avr;
	memset(&addr_avr, 0, sizeof(addr_avr));
	addr_avr.sin6_family = AF_INET6;
	addr_avr.sin6_port = htons(si->port);
	addr_avr.sin6_addr = in6addr_any; // any interface 
	if (bind(s, (struct sockaddr*) &addr_avr, sizeof(addr_avr)) != 0)
		panic("tcp server bind()");
	if (listen(s, 10) != 0)
		panic("tcp server listen()")
	DEBUG("Run %s on TCP port %i\n", si->description, si->port);

	while (1) {
		int *hd1 = (int*) malloc(sizeof(int));
		if ((*hd1 = accept(s, 0, 0)) > -1) {
			if (pthread_create(&child, NULL, si->thread_function, (void *)hd1)) {
				DEBUG("pthread_create %s error\n", si->description);
				free(hd1);
			}
		} else {
			//DEBUG("\ntcp server accept() error.");
			free(hd1);
		}
	}
	// never reach
	free(si);
	close (s);
	pthread_exit(0);
}
